from __future__ import annotations

import logging
from pathlib import Path
from typing import List

import typer
from PIL import Image

from yarrtist.cli.globals import CONTEXT_SETTINGS, OPTIONS, LogLevel
from yarrtist.plotter.core_functions import (
    plot_fe_config,
    plot_module_histo,
)
from yarrtist.utils.utils import (
    create_img_grid,
    data_from_config,
    fill_pdf_summary,
    format_data,
    get_configs_from_connectivity,
    get_geomId_sn_from_config,
    get_moduletype_ordered_chips,
    load_data,
)

log = logging.getLogger("YARRtist")

app = typer.Typer(context_settings=CONTEXT_SETTINGS)


def config_summary_perchip(configs_list, absolute_path, summary_title):
    all_imgs = []

    for chip in configs_list:
        images = []
        _, name = get_geomId_sn_from_config(chip)
        try:
            plotters = plot_fe_config(chip)
            for k in plotters:
                fill_pdf_summary(plotters[k], images)
            chip_img = create_img_grid(images, (2, 2), f"{name}")
            all_imgs.append(chip_img)
        except Exception as e:
            log.warning(f"Failed plotting: {e}")

    if all_imgs:
        summary = [p.convert("RGB") for p in all_imgs]
        summary[0].save(
            f"{absolute_path}/{summary_title}.pdf",
            save_all=True,
            append_images=all_imgs[1:],
        )
        log.info(f"Config summary saved in {absolute_path}/{summary_title}.pdf")
    else:
        log.info("Config summary is empty")


def config_summary(all_data, module_type, ordered_chips, absolute_path, summary_title):
    all_imgs = []

    plot = {}
    plot["data"] = {}

    for n in range(len(all_data)):
        for geomId, test in enumerate(all_data[n]):
            format_data(plot, test, geomId)
        try:
            plotter = plot_module_histo(
                plot, test, module_type, "", [n["name"] for n in ordered_chips]
            )
            fill_pdf_summary(plotter, all_imgs)
        except Exception as e:
            log.warning(f"Failed plotting: {e}")

    if all_imgs:
        summary = [Image.open(p).convert("RGB") for p in all_imgs]
        summary[0].save(
            f"{absolute_path}/{summary_title}.pdf",
            save_all=True,
            append_images=summary[1:],
        )
        log.info(f"Config summary saved in {absolute_path}/{summary_title}.pdf")
    else:
        log.info("Config summary is empty")


@app.command()
def main(
    connectivity_files: List[Path] = OPTIONS["connectivity_files"],
    input_folders: List[Path] = OPTIONS["input_folder"],
    scan_directory: Path = OPTIONS["scan_directory"],
    per_chip: bool = OPTIONS["per_chip"],
    verbosity: LogLevel = OPTIONS["verbosity"],
):
    log.setLevel(verbosity.value)
    log.addHandler(logging.StreamHandler())

    all_modules_configs = []
    all_paths = []

    if connectivity_files:
        for conn in connectivity_files:
            config_list = get_configs_from_connectivity(load_data(conn))
            absolute_path = Path(conn).parent
            all_modules_configs.append(config_list)
            all_paths.append(absolute_path)
    elif input_folders:
        for input_folder in input_folders:
            config_list = [
                str(f.name)
                for f in Path(input_folder).iterdir()
                if f.is_file and "0x" in f.name and ".json" in f.name
            ]
            absolute_path = input_folder
            all_modules_configs.append(config_list)
            all_paths.append(absolute_path)
    else:
        config_list = [
            str(f.name)
            for f in Path(scan_directory).iterdir()
            if f.is_file() and ".after" in f.name
        ]
        absolute_path = scan_directory
        all_modules_configs.append(config_list)
        all_paths.append(absolute_path)
        log.info("Plotting will be done per chip")
        per_chip = True

    for i, (module_config, path) in enumerate(zip(all_modules_configs, all_paths)):
        module_data = []
        for chip_config in module_config:
            module_data.append(load_data(str(path) + "/" + chip_config))

        log.info(f"Plotting module {i + 1}")

        if per_chip:
            config_summary_perchip(module_data, path, f"config_summary_perchip_{i}")
        else:
            module_type, ordered_chips = get_moduletype_ordered_chips(module_data)

            all_data = []
            for chip in ordered_chips:
                all_data.append(data_from_config(chip["config"]))
            all_data = list(map(list, zip(*all_data)))

            config_summary(
                all_data, module_type, ordered_chips, path, f"config_summary_{i}"
            )



from __future__ import annotations

import logging
from pathlib import Path

import typer

from yarrtist.cli.globals import CONTEXT_SETTINGS, OPTIONS, LogLevel
from yarrtist.plotter.core_functions import plot_fe_histo
from yarrtist.utils.utils import load_data

app = typer.Typer(context_settings=CONTEXT_SETTINGS)


@app.command()
def main(
    input_file: Path = OPTIONS["input_file"],
    output_file: Path = OPTIONS["output_file"],
    verbosity: LogLevel = OPTIONS["verbosity"],
):
    log = logging.getLogger("YARRtist")
    log.setLevel(verbosity.value)
    log.addHandler(logging.StreamHandler())

    log.info(f"Plotting single test: {input_file}")

    data = load_data(input_file)
    plotter = plot_fe_histo(data, "", True)

    if output_file is None:
        output_file = Path(input_file).with_suffix(".png")
        log.debug(f"Output file not found, saving in {output_file}")

    plotter.savefig(f"{output_file}")
    plotter.close()

    log.info(f"File saved in {output_file}")



from __future__ import annotations

import sys

from yarrtist._version import __version__

if sys.version_info >= (3, 9):
    from importlib import resources
else:
    import importlib_resources as resources
data = resources.files("yarrtist") / "data"

__all__ = ("__version__", "data")



from __future__ import annotations

from enum import Enum
from pathlib import Path
from typing import List

import typer

CONTEXT_SETTINGS = {"help_option_names": ["-h", "--help"]}


class LogLevel(str, Enum):
    debug = "DEBUG"
    info = "INFO"
    warning = "WARNING"
    error = "ERROR"


OPTIONS = {}

OPTIONS["input_file"]: Path = typer.Option(
    ...,
    "-i",
    "--input-file",
    help="path to the input file.",
    exists=True,
    file_okay=True,
    readable=True,
    resolve_path=True,
)
OPTIONS["output_file"]: Path = typer.Option(
    None,
    "-o",
    "--output-file",
    help="path to the output file.",
    exists=False,
    writable=True,
)
OPTIONS["connectivity_files"]: List[Path] = typer.Option(
    None,
    "-c",
    "--connectivity-files",
    help="path to one or more connectivity file.",
    exists=True,
    file_okay=True,
    readable=True,
    resolve_path=True,
)
OPTIONS["input_folder"]: List[Path] = typer.Option(
    None,
    "-i",
    "--input-folder",
    help="path to the folder with config files.",
    exists=True,
    file_okay=True,
    readable=True,
    resolve_path=True,
)
OPTIONS["scan_directory"]: Path = typer.Option(
    None,
    "-s",
    "--scan-directory",
    help="path to the input YARR scan directory.",
    exists=True,
    readable=True,
    resolve_path=True,
)
OPTIONS["per_chip"]: bool = typer.Option(
    False, "--per-chip", help="divide summary per chip."
)
OPTIONS["config_summary"]: bool = typer.Option(
    True, "--config-summary", help="plot config summary in YARR scan."
)
OPTIONS["failing"]: bool = typer.Option(
    False, "--highlight-failures", help="highlight failures."
)
OPTIONS["verbosity"]: LogLevel = typer.Option(
    LogLevel.info,
    "-v",
    "--verbosity",
    help="Log level [options: DEBUG, INFO (default) WARNING, ERROR]",
)



from __future__ import annotations

import logging
from pathlib import Path
from typing import List

import numpy as np
import typer
from PIL import Image

from yarrtist.cli.globals import CONTEXT_SETTINGS, OPTIONS, LogLevel
from yarrtist.plotter.core_functions import (
    plot_fe_config,
    plot_fe_histo,
    plot_module_histo,
)
from yarrtist.utils.utils import (
    create_img_grid,
    data_from_config,
    fill_pdf_summary,
    format_broken_data,
    format_data,
    get_configs_chips_from_scandir,
    get_configs_from_connectivity,
    get_configs_from_scanlog,
    get_moduletype_ordered_chips,
    get_tests_chips_from_scandir,
    load_data,
)

log = logging.getLogger("YARRtist")

app = typer.Typer(context_settings=CONTEXT_SETTINGS)


################## plot module chip by chip ######################


def summary_perchip(scan_dir, tests_names, chip_names, summary_title, failing):
    all_imgs = []
    for c in chip_names:
        log.info(f"Plotting for FE {c}")
        for t in tests_names:
            try:
                log.debug(f"Plotting {t}")
                data = load_data(f"{scan_dir}/{c}_{t}.json")
            except Exception:
                log.debug("Test not found for this chip, skipping")
                continue
            try:
                plotter = plot_fe_histo(data, c)
                fill_pdf_summary(plotter, all_imgs)
                if failing and data.get("Name") in [
                    "NoiseMap-0",
                    "ThresholdMap-0",
                    "OccupancyMap",
                    "NoiseOccupancy",
                    "Occupancy",
                ]:
                    data["Name"] = data["Name"] + " - Failures"
                    plotter = plot_fe_histo(data, c, True)
                    fill_pdf_summary(plotter, all_imgs)
            except Exception as e:
                log.warning(f"Failed plotting {t}: {e}")

    if all_imgs:
        summary = [Image.open(p).convert("RGB") for p in all_imgs]

        summary[0].save(
            f"{scan_dir}/{summary_title}.pdf",
            save_all=True,
            append_images=summary[1:],
        )
        log.info(f"Plot summary saved in {scan_dir}/{summary_title}.pdf")
    else:
        log.info("Plot summary is empty")


def config_summary_perchip(scan_dir, files_list, chip_names, summary_title):
    all_imgs = []

    for chip in chip_names:
        configs = {
            "before": [f for f in files_list if ".before" in f and chip in f],
            "after": [f for f in files_list if ".after" in f and chip in f],
        }
        for ba, config in configs.items():
            if len(config) == 0:
                continue
            images = []
            config_data = load_data(f"{scan_dir}/{config[0]}")
            try:
                plotters = plot_fe_config(config_data)
                for k in plotters:
                    fill_pdf_summary(plotters[k], images)
                chip_img = create_img_grid(images, (2, 2), f"{chip} ({ba})")
                all_imgs.append(chip_img)
            except Exception:
                log.warning("Failed to plot: {e}")

    if all_imgs:
        summary = [p.convert("RGB") for p in all_imgs]
        summary[0].save(
            f"{scan_dir}/{summary_title}.pdf",
            save_all=True,
            append_images=all_imgs[1:],
        )
        log.info(f"Config summary saved in {scan_dir}/{summary_title}.pdf")
    else:
        log.info("Config summary is empty")


################### summary of module from YARR scan directory ###################
def module_summary(
    files_list,
    tests_names,
    module_type,
    ordered_chips,
    scan_dir,
    summary_title,
    failing,
):
    images = []

    for test in tests_names:
        plot = {}
        plot["data"] = {}

        log.info(f"Plotting {test}")
        not_found = 0

        for geomId, item in enumerate(ordered_chips):
            chip = item.get("name")
            ctype = item.get("type")
            file_name = f"{chip}_{test}.json"
            log.debug(f"Plotting for chip {geomId + 1}: {chip}")
            if file_name not in files_list:
                log.debug(f"Chip {chip} not found, filling with 0s")
                not_found += 1
                format_broken_data(plot, test, ctype, geomId)
                continue

            data = load_data(f"{scan_dir}/{file_name}")
            format_data(plot, data, geomId)

        if (not_found == 4 and module_type == "Quad") or (
            not_found == 3 and module_type == "Triplet"
        ):
            log.debug("Skipping whole test for this module")
            continue
        try:
            plotter = plot_module_histo(
                plot, data, module_type, "", [n.get("name") for n in ordered_chips]
            )
            fill_pdf_summary(plotter, images)
            if failing and plot.get("title") in [
                "NoiseMap-0",
                "ThresholdMap-0",
                "OccupancyMap",
                "NoiseOccupancy",
                "Occupancy",
            ]:
                plot["title"] = plot.get("title") + " - Failures"
                plotter = plot_module_histo(
                    plot,
                    data,
                    module_type,
                    "",
                    [n.get("name") for n in ordered_chips],
                    True,
                )
                fill_pdf_summary(plotter, images)
        except Exception:
            log.warning("Failed to plot {test}: {e}")

    if images:
        summary = [Image.open(p).convert("RGB") for p in images]
        summary[0].save(
            f"{scan_dir}/{summary_title}.pdf",
            save_all=True,
            append_images=summary[1:],
        )
        log.info(f"Plot summary saved in {scan_dir}/{summary_title}.pdf")
    else:
        log.info("Plot summary is empty")


################### summary of module config from YARR scan directory ###################
def module_config_summary(
    files_list, module_type, ordered_chips, scan_dir, summary_title
):
    all_data_before = []
    all_data_after = []
    all_imgs = []
    for _geomId, chip in enumerate(ordered_chips):
        ba_configs = [f for f in files_list if chip.get("name") in f]
        if len(ba_configs) == 0:
            all_data_before.append([{}] * 5)
            all_data_after.append([{}] * 5)
            continue
        for conf in ba_configs:
            if ".before" in conf:
                all_data_before.append(
                    data_from_config(load_data(f"{scan_dir}/{conf}"))
                )
            elif ".after" in conf:
                all_data_after.append(data_from_config(load_data(f"{scan_dir}/{conf}")))

    all_data_before = list(map(list, zip(*all_data_before)))
    all_data_after = list(map(list, zip(*all_data_after)))

    for n in range(len(all_data_after)):
        plot_before = {}
        plot_after = {}
        plot_before["data"] = {}
        plot_after["data"] = {}
        images = []
        for geomId, (tb, ta) in enumerate(zip(all_data_before[n], all_data_after[n])):
            if not tb or not ta:
                plot_before["data"].update({geomId: np.zeros((400, 384))})
                plot_after["data"].update({geomId: np.zeros((400, 384))})
                continue
            format_data(plot_before, tb, geomId)
            plot_before["title"] = plot_before["title"] + " (before)"
            format_data(plot_after, ta, geomId)
            plot_after["title"] = plot_after["title"] + " (after)"
        try:
            plotter = plot_module_histo(
                plot_before, tb, module_type, "", [n.get("name") for n in ordered_chips]
            )
            fill_pdf_summary(plotter, images)
            plotter = plot_module_histo(
                plot_after, ta, module_type, "", [n.get("name") for n in ordered_chips]
            )
            fill_pdf_summary(plotter, images)
            all_imgs.append(create_img_grid(images, (1, 2)))
        except Exception as e:
            log.warning(f"Failed plotting: {e}")

    if all_imgs:
        summary = [p.convert("RGB") for p in all_imgs]

        summary[0].save(
            f"{scan_dir}/{summary_title}.pdf",
            save_all=True,
            append_images=all_imgs[1:],
        )
        log.info(f"Config summary saved in {scan_dir}/{summary_title}.pdf")
    else:
        log.info("Config summary is empty")


@app.command()
def main(
    connectivity_files: List[Path] = OPTIONS["connectivity_files"],
    input_folders: List[Path] = OPTIONS["input_folder"],
    scan_directory: Path = OPTIONS["scan_directory"],
    per_chip: bool = OPTIONS["per_chip"],
    config_summary: bool = OPTIONS["config_summary"],
    failing: bool = OPTIONS["failing"],
    verbosity: LogLevel = OPTIONS["verbosity"],
):
    log.setLevel(verbosity.value)
    log.addHandler(logging.StreamHandler())

    test_files_list, tests_names, test_chip_names = get_tests_chips_from_scandir(
        scan_directory
    )
    config_files_list, config_chip_names = get_configs_chips_from_scandir(
        scan_directory
    )

    if per_chip:
        log.info("Plotting module scan summary, divided per chip")

        summary_perchip(
            scan_directory, tests_names, test_chip_names, "summary_perchip", failing
        )

        if config_summary:
            log.info("Plotting module config summary, divided per chip")

            config_summary_perchip(
                scan_directory,
                config_files_list,
                config_chip_names,
                "config_summary_perchip",
            )
    else:
        log.info("Plotting module scan summary")

        if not (connectivity_files or input_folders):
            module_configs = get_configs_from_scanlog(
                load_data(f"{scan_directory}/scanLog.json")
            )
        else:
            module_configs = []
            if connectivity_files:
                for connectivity in connectivity_files:
                    chip_configs = []
                    config_dir = Path(connectivity).parent
                    connectivity_data = load_data(connectivity)
                    for chip in get_configs_from_connectivity(connectivity_data):
                        chip_configs.append(f"{config_dir}/{chip}")
                    module_configs.append(chip_configs)
            if input_folders:
                for input_folder in input_folders:
                    chip_configs = [
                        str(input_folder) + "/" + str(f.name)
                        for f in Path(input_folder).iterdir()
                        if f.is_file and "0x" in f.name and ".json" in f.name
                    ]
                    module_configs.append(chip_configs)

        for i, module in enumerate(module_configs):
            log.info(f"Plotting for module {i + 1}")
            module_data = []
            for chip in module:
                module_data.append(load_data(f"{chip}"))

            module_type, ordered_chips = get_moduletype_ordered_chips(module_data)
            log.debug(f"Module is a {module_type}")
            # log.debug(f"Ordered chips in module {ordered_chips}")

            module_summary(
                test_files_list,
                tests_names,
                module_type,
                ordered_chips,
                scan_directory,
                f"module_summary_{i}",
                failing,
            )
            if config_summary:
                log.info("Plotting module config summary")
                module_config_summary(
                    config_files_list,
                    module_type,
                    ordered_chips,
                    scan_directory,
                    f"module_config_summary_{i}",
                )



from __future__ import annotations

import contextlib
import importlib.resources
import json
import logging
from io import BytesIO
from pathlib import Path

import numpy as np
from PIL import Image, ImageDraw, ImageFont
from scipy.stats import norm

import yarrtist.utils.fonts

logger = logging.getLogger("YARRtist")


def get_tests_chips_from_scandir(scan_dir):
    tests_names = []
    chip_names = []

    files_list = [
        f.name
        for f in Path(scan_dir).iterdir()
        if f.is_file()
        and "0x" in f.name
        and all(layer not in f.name for layer in ["L0", "L1", "L2", "R0"])
        and ".json" in f.name
    ]

    for f in files_list:
        try:
            c = f.split("_", 1)
            ct = c[1]
            cn = c[0]
            tests_names.append(ct.split(".")[0])
            chip_names.append(cn)
        except IndexError:
            continue

    return files_list, list(set(tests_names)), list(set(chip_names))


def get_configs_chips_from_scandir(scan_dir):
    chip_names = []

    files_list = [
        f.name
        for f in Path(scan_dir).iterdir()
        if (f.is_file() and ".before" in f.name) or (f.is_file() and ".after" in f.name)
    ]

    for f in files_list:
        try:
            cn = f.split("_", 1)[0]
            chip_names.append(cn)
        except IndexError:
            continue

    return files_list, list(set(chip_names))


def load_data(input_data):
    with Path(input_data).open() as file:
        return json.load(file)


def get_configs_from_scanlog(scanlog):
    all_paths = []
    common_path = ""

    logger.info("Trying to find chips configuration files")

    for conn in scanlog.get("connectivity"):
        module_paths = []
        for chip in conn.get("chips"):
            chip_path = Path(chip.get("__config_path__") or chip.get("config"))
            if Path(common_path + str(chip_path)).exists():
                logger.debug(f"Found chip config: {common_path + str(chip_path)}")
                module_paths.append(common_path + str(chip_path))
                continue
            for path in Path("/").rglob(chip_path.name):
                # logger.debug(path)
                if str(path).endswith(str(chip_path)):
                    logger.debug(f"Found chip config: {path}")
                    module_paths.append(str(path))
                    try:
                        common_path = str(path).removesuffix(str(chip_path))
                    except Exception:
                        common_path = str(path)[: -len(str(chip_path))]
                    break
        all_paths.append(module_paths)

    return all_paths


def get_chip_type_from_config(config):
    chiptype = ""
    try:
        chiptype = next(iter(config.keys()))
    except IndexError:
        logger.error("One of your chip configuration files is empty")

    if chiptype not in {"RD53B", "ITKPIXV2"}:
        logger.warning(
            "Chip name in configuration not one of expected chip names (RD53B or ITKPIXV2)"
        )
    return chiptype


def get_geomId_sn_from_config(config_data):
    chip_type = get_chip_type_from_config(config_data)
    config_parameter = config_data[chip_type].get("Parameter", []) if chip_type else {}
    chip_id = config_parameter.get("ChipId", 0)
    chip_name = config_parameter.get("Name", "")

    if chip_id == 12:
        return 1, chip_name
    if chip_id == 13:
        return 2, chip_name
    if chip_id == 14:
        return 3, chip_name
    if chip_id == 15:
        return 4, chip_name
    return chip_id, chip_name


def get_moduletype_ordered_chips(module):
    if len(module) == 3:
        module_type = "Triplet"
    elif len(module) == 4:
        module_type = "Quad"
    else:
        logger.error("Length doesn't match a known module type")
        return None, None

    ordered_chips = [{}] * len(module)
    for chip in module:
        geomId, name = get_geomId_sn_from_config(chip)
        ctype = get_chip_type_from_config(chip)
        ordered_chips[geomId - 1] = {
            "name": name,
            "type": ctype,
            "config": chip,
        }

    return module_type, ordered_chips


def format_broken_data(plot, test, ctype, geomId):
    if "InjVcalDiff_Map" in test:
        plot["data"].update(
            {
                geomId: np.zeros((153600, 101))
                if ctype == "ITKPIXV2"
                else np.zeros((153600, 61))
            }
        )
    elif "InjVcalDiff" in test:
        plot["data"].update(
            {geomId: np.zeros((101, 49)) if ctype == "ITKPIXV2" else np.zeros((61, 49))}
        )
    else:
        plot["data"].update({geomId: np.zeros((400, 384))})


def format_data(plot, data, geomId):
    plot["type"] = data.get("Type")
    plot["title"] = data.get("Name")

    if plot.get("type") == "Histo2d":
        data_arr = np.array(data.get("Data"))
        if None in data_arr:
            data_arr[np.equal(data_arr, None)] = 0
        plot["data"].update({geomId: data_arr})
    else:
        plot["data"].update({geomId: data})


def fill_pdf_summary(plotter, all_imgs, dpi=300):
    buf = BytesIO()
    plotter.savefig(buf, format="png", dpi=dpi)
    all_imgs.append(buf)
    buf.seek(0)
    try:
        plotter.close()
    except Exception:
        contextlib.suppress(Exception)


def create_img_grid(images_buf, grid, text=None):
    spacing = 10
    bkg_color = (255, 255, 255)

    images = [Image.open(p) for p in images_buf]

    max_width = max(img.width for img in images)
    max_height = max(img.height for img in images)

    rows, cols = grid
    total_width = cols * max_width + (cols - 1) * spacing
    total_height = rows * max_height + (rows - 1) * spacing

    text_height = 0
    if text is not None:
        try:
            try:
                with (
                    importlib.resources.files(yarrtist.utils.fonts)
                    .joinpath("Roboto_Condensed-BoldItalic.ttf")
                    .open("rb")
                ) as f:
                    font = ImageFont.truetype(f, 65)
            except Exception:
                with importlib.resources.path(  # pylint: disable=deprecated-method
                    "yarrtist.utils.fonts", "Roboto_Condensed-BoldItalic.ttf"
                ) as font_path:
                    font = ImageFont.truetype(str(font_path), 65)
        except OSError as e:
            logger.debug(e)
            font = ImageFont.load_default()
        bbox = font.getbbox(text)
        text_width = bbox[2] - bbox[0]
        text_height = bbox[3] - bbox[1]
        total_height += text_height + spacing

    combined_image = Image.new("RGB", (total_width, total_height), bkg_color)

    for idx, img in enumerate(images):
        row, col = divmod(idx, cols)
        x = col * (max_width + spacing)
        y = row * (max_height + spacing)
        combined_image.paste(img, (x, y))

    if text is not None:
        draw = ImageDraw.Draw(combined_image)
        text_x = (total_width - text_width) // 2
        text_y = total_height - text_height - 150 * spacing
        draw.text((text_x, text_y), text, fill="black", font=font)

    return combined_image


def get_configs_from_connectivity(conn_data):
    configs = []

    for c in conn_data.get("chips", []):
        configs.append(c.get("config"))

    return configs


def data_from_config(config_data):
    plot_data = []

    chip_type = get_chip_type_from_config(config_data)
    config_pixel = config_data[chip_type].get("PixelConfig", []) if chip_type else []

    if config_pixel:
        pixelDataMap = {
            k: [dic[k] for dic in config_pixel] for k in config_pixel[0] if k != "Col"
        }

        for key, data in pixelDataMap.items():
            plot_data.append(
                {
                    "Data": data,
                    "Entries": np.sum(np.array(data)),
                    "Name": key + "-Map",
                    "Overflow": 0.0,
                    "Type": "Histo2d",
                    "Underflow": 0.0,
                    "x": {
                        "AxisTitle": "Column",
                        "Bins": 400,
                        "High": 400.5,
                        "Low": 0.5,
                    },
                    "y": {"AxisTitle": "Row", "Bins": 384, "High": 384.5, "Low": 0.5},
                    "z": {"AxisTitle": key},
                }
            )

            if key == "TDAC":
                flat = np.array(data).flatten()
                bins = np.linspace(-15.5, 15.5, 32)
                hist, _edges = np.histogram(flat, bins=bins)
                plot_data.append(
                    {
                        "Data": hist,
                        "Entries": np.sum(flat),
                        "Name": "TADC-Dist",
                        "Overflow": 0.0,
                        "Type": "Histo1d",
                        "Underflow": 0.0,
                        "x": {
                            "AxisTitle": "TDAC",
                            "Bins": 31,
                            "High": 15.5,
                            "Low": -15.5,
                        },
                        "y": {"AxisTitle": "Number of Pixels"},
                        "z": {"AxisTitle": "z"},
                    }
                )

    else:
        logger.debug("No data for chip")
        for key in ["Enable", "Hitbus", "InjEn", "TDAC"]:
            plot_data.append(
                {
                    "Data": np.zeros((400, 384)),
                    "Entries": 0,
                    "Name": key + "-Map",
                    "Overflow": 0.0,
                    "Type": "Histo2d",
                    "Underflow": 0.0,
                    "x": {
                        "AxisTitle": "Column",
                        "Bins": 400,
                        "High": 400.5,
                        "Low": 0.5,
                    },
                    "y": {"AxisTitle": "Row", "Bins": 384, "High": 384.5, "Low": 0.5},
                    "z": {"AxisTitle": key},
                }
            )

    return plot_data


def fit_Histo1d(freq, bins):
    centers = []
    for i in range(len(bins)):
        centers.append(0.5 * (bins[i] + bins[i - 1]))

    data = np.repeat(centers, freq)

    return norm.fit(data)



from __future__ import annotations

from pathlib import Path

import pytest
from typer.testing import CliRunner

from yarrtist.cli import app


@pytest.fixture
def config_path():
    return Path() / "src" / "yarrtist" / "data" / "example_configs"


@pytest.fixture
def scan_path():
    return Path() / "src" / "yarrtist" / "data" / "example_scans"


@pytest.fixture
def runner():
    return CliRunner(mix_stderr=False)


def test_single_plot(runner, scan_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-single-test",
            "-i",
            scan_path.joinpath("010069_std_analogscan/0x22e4d_OccupancyMap.json"),
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "File saved in" in caplog.text


def test_single_config(runner, config_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-single-config",
            "-i",
            config_path.joinpath("20UPGM23211219/L2_warm/0x22e8c_L2_warm.json"),
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "File saved in" in caplog.text


def test_scan(runner, scan_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-scan",
            "-s",
            scan_path.joinpath("010290_std_analogscan"),
            "--highlight-failures",
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Plot summary saved" in caplog.text
    assert "Config summary saved" in caplog.text


def test_scan_perchip(runner, scan_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-scan",
            "-s",
            scan_path.joinpath("010290_std_analogscan"),
            "--highlight-failures",
            "--per-chip",
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Plot summary saved" in caplog.text
    assert "Config summary saved" in caplog.text


def test_config(runner, config_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-connectivity",
            "-i",
            config_path.joinpath("20UPGM23211219/L2_warm"),
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Config summary saved in" in caplog.text


def test_config_perchip(runner, config_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-connectivity",
            "-c",
            config_path.joinpath("20UPGM23211219/20UPGM23211219_L2_warm.json"),
            "--per-chip",
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Config summary saved in" in caplog.text


def test_broken(runner, scan_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-scan",
            "-s",
            scan_path.joinpath("010069_std_analogscan"),
            "--highlight-failures",
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Plot summary saved" in caplog.text
    assert "Config summary saved" in caplog.text


def test_broken_perchip(runner, scan_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-scan",
            "-s",
            scan_path.joinpath("010069_std_analogscan"),
            "--highlight-failures",
            "--per-chip",
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Plot summary saved" in caplog.text
    assert "Config summary saved" in caplog.text


def test_triplet_scan(runner, scan_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-scan",
            "-s",
            scan_path.joinpath("008847_std_analogscan"),
            "--highlight-failures",
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Plot summary saved" in caplog.text
    assert "Config summary saved" in caplog.text


def test_triplet_scan_perchip(runner, scan_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-scan",
            "-s",
            scan_path.joinpath("008847_std_analogscan"),
            "--highlight-failures",
            "--per-chip",
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Plot summary saved" in caplog.text
    assert "Config summary saved" in caplog.text


def test_triplet_config(runner, config_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-connectivity",
            "-c",
            config_path.joinpath("20UPIM52204106/20UPIM52204106_R0.5_warm.json"),
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Config summary saved in" in caplog.text


def test_triplet_config_perchip(runner, config_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-connectivity",
            "-i",
            config_path.joinpath("20UPIM52204106/R0.5_warm"),
            "--per-chip",
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Config summary saved in" in caplog.text


def test_combined(runner, scan_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-scan",
            "-s",
            scan_path.joinpath("combined_analog_scan"),
            "--highlight-failures",
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Plot summary saved" in caplog.text
    assert "Config summary saved" in caplog.text


def test_combined_config_option(runner, config_path, scan_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-scan",
            "-i",
            config_path.joinpath("20UPGM23211219/L2_warm"),
            "-i",
            config_path.joinpath("20UPGM23211190/L2_warm"),
            "-s",
            scan_path.joinpath("combined_analog_scan"),
            "--highlight-failures",
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Plot summary saved" in caplog.text
    assert "Config summary saved" in caplog.text


def test_combined_conn_option(runner, config_path, scan_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-scan",
            "-c",
            config_path.joinpath("20UPGM23211219/20UPGM23211219_L2_warm.json"),
            "-c",
            config_path.joinpath("20UPGM23211190/20UPGM23211190_L2_warm.json"),
            "-s",
            scan_path.joinpath("combined_analog_scan"),
            "--highlight-failures",
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Plot summary saved" in caplog.text
    assert "Config summary saved" in caplog.text


def test_combined_perchip(runner, scan_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-scan",
            "-s",
            scan_path.joinpath("combined_analog_scan"),
            "--highlight-failures",
            "--per-chip",
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Plot summary saved" in caplog.text
    assert "Config summary saved" in caplog.text


def test_conn_combined_config_option(runner, config_path, caplog):
    result = runner.invoke(
        app,
        args=[
            "plots",
            "plot-connectivity",
            "-i",
            config_path.joinpath("20UPGM23211219/L2_warm"),
            "-i",
            config_path.joinpath("20UPGM23211190/L2_warm"),
            "-v",
            "DEBUG",
        ],
        catch_exceptions=False,
    )

    assert result.exit_code == 0
    assert "Config summary" in caplog.text



from __future__ import annotations

import logging
from io import BytesIO
from pathlib import Path

import typer

from yarrtist.cli.globals import CONTEXT_SETTINGS, OPTIONS, LogLevel
from yarrtist.plotter.core_functions import plot_fe_config
from yarrtist.utils.utils import create_img_grid, load_data

app = typer.Typer(context_settings=CONTEXT_SETTINGS)


@app.command()
def main(
    input_file: Path = OPTIONS["input_file"],
    output_file: Path = OPTIONS["output_file"],
    verbosity: LogLevel = OPTIONS["verbosity"],
):
    log = logging.getLogger("YARRtist")
    log.setLevel(verbosity.value)
    log.addHandler(logging.StreamHandler())

    log.info(f"Plotting config file: {input_file}")

    data = load_data(input_file)
    plotters = plot_fe_config(data)

    images = []

    for k in plotters:
        buf = BytesIO()
        plotters[k].savefig(buf, format="png")
        images.append(buf)
        buf.seek(0)
    test_img = create_img_grid(images, (2, 2))

    if output_file is None:
        output_file = Path(input_file).with_suffix(".png")
        log.debug(f"Output file not found, saving in {output_file}")

    test_img.save(f"{output_file}")

    log.info(f"File saved in {output_file}")



from __future__ import annotations

import contextlib
import logging

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.colors import LogNorm
from scipy.stats import norm

from yarrtist.utils.utils import fit_Histo1d, get_chip_type_from_config

logger = logging.getLogger("YARRtist")

np.set_printoptions(threshold=np.inf)


def plot_fe_config(config_data):
    returnMap = {}

    chip_type = get_chip_type_from_config(config_data)
    config_pixel = config_data[chip_type].get("PixelConfig", []) if chip_type else []

    if config_pixel:
        pixelDataMap = {
            k: [dic[k] for dic in config_pixel] for k in config_pixel[0] if k != "Col"
        }

        for key, data in pixelDataMap.items():
            arr = np.array(data)

            vmin, vmax = 0, 1
            ticks = [0, 1]
            if key.lower() == "tdac":
                vmin, vmax = -15, 15
                ticks = [-15, -10, -5, 0, 5, 10, 15]

            cmap = plt.get_cmap("magma", (vmax - vmin) + 1)

            fig, ax = plt.subplots()
            cax = ax.imshow(arr.T, origin="lower", cmap=cmap, vmin=vmin, vmax=vmax)
            ax.set_title(key, fontsize=22)
            ax.set_xlabel("Column", fontsize=18)
            ax.set_ylabel("Row", fontsize=18)
            fig.tight_layout()

            cb = fig.colorbar(cax, label="Value", ax=ax)
            cb.set_ticks(ticks)
            for t in cb.ax.get_yticklabels():
                t.set_fontsize(12)

            returnMap[key] = fig

            plt.close(fig)

    return returnMap


def plot_fe_histo(data, chip_name="", failing_pixels=False):
    if data.get("Type") == "Histo1d":
        bins = np.linspace(
            data.get("x").get("Low"),
            data.get("x").get("High"),
            data.get("x").get("Bins"),
        )

        fit_label = f"(Uf: {data.get('Underflow')},\n Of: {data.get('Overflow')})"
        if data.get("Name") == "ThresholdDist-0" or data.get("Name") == "NoiseDist-0":
            mu, sigma = fit_Histo1d(data.get("Data"), bins)
            y = norm.pdf(bins, mu, sigma) * np.sum(data.get("Data")) * np.diff(bins)[0]
            plt.plot(bins, y, "r--", label="fit")
            fit_label = f"(µ: {mu:.2f}, σ: {sigma:.2f},\n Uf: {data.get('Underflow')},\n Of: {data.get('Overflow')})"

        plt.hist(bins, len(bins), weights=data.get("Data"), label=f"{fit_label}")
        plt.legend()

    elif data.get("Type") == "Histo2d":
        arr = np.array(data.get("Data"))
        if None in arr:
            arr[np.equal(arr, None)] = 0

        is_config = np.all(np.isin(arr, [0, 1]))

        ma = np.ma.MaskedArray(arr, arr <= 0)
        min_nonzero_value = np.ma.min(ma)
        max_nonzero_value = np.ma.max(ma)
        min_max_ratio = max_nonzero_value / (min_nonzero_value + 1.0e-19)

        set_log_scale = min_nonzero_value > 0 and min_max_ratio > 100

        if set_log_scale:
            arr += min_nonzero_value * 1.0e-2
            plt.imshow(
                arr.T,
                origin="lower",
                cmap="turbo",
                norm=LogNorm(min_nonzero_value * 0.5, max_nonzero_value * 2),
            )
        else:
            plt.imshow(
                arr.T, origin="lower", cmap="turbo" if not is_config else "magma"
            )

        cb = plt.colorbar(label=data.get("z").get("AxisTitle"))
        for t in cb.ax.get_yticklabels():
            t.set_fontsize(12)

        if failing_pixels:
            valid_rows = ~(arr == np.zeros((400, 384))).all(axis=1)
            valid_cols = ~(arr == np.zeros((400, 384))).all(axis=0)

            valid_mask = np.zeros((400, 384), dtype=bool)

            valid_mask = np.outer(valid_rows, valid_cols)

            mean_occ = np.mean(arr[valid_mask])

            is_low = (arr < mean_occ / 2) & valid_mask
            is_high = (arr > mean_occ * 2) & valid_mask

            x_low, y_low = np.where(is_low)
            x_high, y_high = np.where(is_high)

            plt.scatter(x_low, y_low, s=3, c="black", marker="s", label="Low Occ.")
            plt.scatter(x_high, y_high, s=3, c="fuchsia", marker="s", label="High Occ.")
            plt.legend(fontsize=5)

    else:
        logger.error("unexpected data type")
        return None

    plt.title(data.get("Name") + f" {chip_name}", fontsize=22)
    plt.xlabel(data.get("x").get("AxisTitle"), fontsize=18)
    plt.ylabel(data.get("y").get("AxisTitle"), fontsize=18)
    plt.tight_layout()

    return plt


def plot_module_histo(
    plot, data, moduleType, serialNumber="", fe_serialNumbers=None, failing_pixels=False
):
    if fe_serialNumbers is None:
        fe_serialNumbers = ["", "", "", ""]

    if plot.get("type") == "Histo2d" and plot.get("title") != "InjVcalDiff":
        if moduleType == "Triplet":
            plt.figure(figsize=(18, 16))
            ax = plt.gca()
            ax.tick_params(axis="both", which="major", labelsize=14)

            module = np.concatenate(
                (
                    plot.get("data")[0].T,
                    plot.get("data")[1].T,
                    plot.get("data")[2].T,
                ),
                axis=1,
            )

            is_config = np.all(np.isin(module, [0, 1]))

            if failing_pixels:
                valid_rows_1 = ~(module[:384, :400] == np.zeros((384, 400))).all(axis=1)
                valid_cols_1 = ~(module[:384, :400] == np.zeros((384, 400))).all(axis=0)
                valid_rows_2 = ~(module[:384, 400:800] == np.zeros((384, 400))).all(
                    axis=1
                )
                valid_cols_2 = ~(module[:384, 400:800] == np.zeros((384, 400))).all(
                    axis=0
                )
                valid_rows_3 = ~(module[:384, 800:1200] == np.zeros((384, 400))).all(
                    axis=1
                )
                valid_cols_3 = ~(module[:384, 800:1200] == np.zeros((384, 400))).all(
                    axis=0
                )

                valid_mask = np.zeros((384, 1200), dtype=bool)

                valid_mask[:384, :400] = np.outer(valid_rows_1, valid_cols_1)
                valid_mask[:384, 400:800] = np.outer(valid_rows_2, valid_cols_2)
                valid_mask[:384, 800:1200] = np.outer(valid_rows_3, valid_cols_3)

                mean_occ = np.mean(module[valid_mask])

                is_low = (module < mean_occ / 2) & valid_mask
                is_high = (module > mean_occ * 2) & valid_mask

                y_low, x_low = np.where(is_low)
                y_high, x_high = np.where(is_high)

            module = np.array(module, dtype=float)
            ma = np.ma.MaskedArray(module, module <= 0)
            min_nonzero_value = np.ma.min(ma)
            max_nonzero_value = np.ma.max(ma)
            min_max_ratio = max_nonzero_value / (min_nonzero_value + 1.0e-19)

            set_log_scale = min_nonzero_value > 0 and min_max_ratio > 100

            if set_log_scale:
                module += min_nonzero_value * 1.0e-2
                plt.imshow(
                    module,
                    origin="lower",
                    cmap="turbo",
                    norm=LogNorm(min_nonzero_value * 0.5, max_nonzero_value * 2),
                )
            else:
                plt.imshow(
                    module, origin="lower", cmap="turbo" if not is_config else "magma"
                )

            plt.title(
                "Triplet Module: " + serialNumber + " " + plot.get("title"),
                fontsize=18,
            )
            ax.text(
                200,
                -20,
                "FE1: " + fe_serialNumbers[0],
                horizontalalignment="center",
                verticalalignment="top",
                size=10,
                color="blue",
            )
            ax.text(
                600,
                -20,
                "FE2: " + fe_serialNumbers[1],
                horizontalalignment="center",
                verticalalignment="top",
                size=10,
                color="blue",
            )
            ax.text(
                1000,
                -20,
                "FE3: " + fe_serialNumbers[2],
                horizontalalignment="center",
                verticalalignment="top",
                size=10,
                color="blue",
            )

            if module.shape == (384, 1200):
                ax.set_xticks(np.arange(0, 1200 + 1, 400))
                ax.set_yticks(np.arange(0, 384 + 1, 384))
                ax.set_xticks(np.arange(0, 1200 + 1, 16), minor=True)
                ax.set_yticks(np.arange(0, 384 + 1, 16), minor=True)

                ax.grid(which="major", color="black", linestyle="solid", linewidth=0.5)

            plt.xlabel("Column", fontsize=18)
            plt.ylabel("Row", fontsize=18)
            cb = plt.colorbar(
                label=data.get("z").get("AxisTitle"), orientation="horizontal"
            )
            for t in cb.ax.get_yticklabels():
                t.set_fontsize(12)

            if failing_pixels:
                plt.scatter(x_low, y_low, s=3, c="black", marker="s", label="Low Occ.")
                plt.scatter(
                    x_high, y_high, s=3, c="fuchsia", marker="s", label="High Occ."
                )
                plt.legend(fontsize=5)

        else:
            up = np.zeros((400, 768))
            logger.debug(f"{plot.get('data')[0].shape}, {plot.get('data')[3].shape}")
            try:
                up = np.concatenate(
                    (
                        np.flip(np.flip(plot.get("data")[0], axis=0), axis=1),
                        plot.get("data")[3],
                    ),
                    axis=1,
                )
            except Exception:
                try:
                    up = np.concatenate(
                        (
                            np.flip(np.flip(plot.get("data")[0], axis=0), axis=1),
                            np.zeros((400, 384)),
                        ),
                        axis=1,
                    )
                except Exception:
                    with contextlib.suppress(Exception):
                        up = np.concatenate(
                            (np.zeros((400, 384)), plot.get("data")[3]), axis=1
                        )

            down = np.zeros((400, 768))
            logger.debug(f"{plot.get('data')[1].shape}, {plot.get('data')[2].shape}")
            try:
                down = np.concatenate(
                    (
                        np.flip(np.flip(plot.get("data")[1], axis=0), axis=1),
                        plot.get("data")[2],
                    ),
                    axis=1,
                )
            except Exception:
                try:
                    down = np.concatenate(
                        (
                            np.flip(np.flip(plot.get("data")[1], axis=0), axis=1),
                            np.zeros((400, 384)),
                        ),
                        axis=1,
                    )
                except Exception:
                    with contextlib.suppress(Exception):
                        down = np.concatenate(
                            (np.zeros((400, 384)), plot.get("data")[1]), axis=1
                        )

            try:
                module = np.concatenate((down, up), axis=0)
            except Exception as e:
                logger.info(f"Plotting failed: str{e}")

            is_config = np.all(np.isin(module, [0, 1]))

            if failing_pixels:
                valid_rows_1 = ~(module[:400, :384] == np.zeros((400, 384))).all(axis=1)
                valid_cols_1 = ~(module[:400, :384] == np.zeros((400, 384))).all(axis=0)
                valid_rows_2 = ~(module[400:800, :384] == np.zeros((400, 384))).all(
                    axis=1
                )
                valid_cols_2 = ~(module[400:800, :384] == np.zeros((400, 384))).all(
                    axis=0
                )
                valid_rows_3 = ~(module[:400, 384:768] == np.zeros((400, 384))).all(
                    axis=1
                )
                valid_cols_3 = ~(module[:400, 384:768] == np.zeros((400, 384))).all(
                    axis=0
                )
                valid_rows_4 = ~(module[400:800, 384:768] == np.zeros((400, 384))).all(
                    axis=1
                )
                valid_cols_4 = ~(module[400:800, 384:768] == np.zeros((400, 384))).all(
                    axis=0
                )

                valid_mask = np.zeros((800, 768), dtype=bool)

                valid_mask[:400, :384] = np.outer(valid_rows_1, valid_cols_1)
                valid_mask[400:800, :384] = np.outer(valid_rows_2, valid_cols_2)
                valid_mask[:400, 384:768] = np.outer(valid_rows_3, valid_cols_3)
                valid_mask[400:800, 384:768] = np.outer(valid_rows_4, valid_cols_4)

                mean_occ = np.mean(module[valid_mask])

                is_low = (module < mean_occ / 2) & valid_mask
                is_high = (module > mean_occ * 2) & valid_mask

                y_low, x_low = np.where(is_low)
                y_high, x_high = np.where(is_high)

            module = module.astype("float")
            ma = np.ma.MaskedArray(module, module <= 0)
            min_nonzero_value = np.ma.min(ma)
            max_nonzero_value = np.ma.max(ma)
            min_max_ratio = max_nonzero_value / (min_nonzero_value)

            set_log_scale = min_nonzero_value > 0 and min_max_ratio > 100

            plt.figure(figsize=(12, 9))

            if set_log_scale:
                module += min_nonzero_value * 1.0e-2
                plt.imshow(
                    module,
                    origin="lower",
                    cmap="turbo",
                    extent=[-384, 384, -400, 400],
                    norm=LogNorm(min_nonzero_value * 0.5, max_nonzero_value * 2),
                )
            else:
                plt.imshow(
                    module,
                    origin="lower",
                    cmap="turbo" if not is_config else "magma",
                    extent=[-384, 384, -400, 400],
                )

            ax = plt.gca()
            ax.tick_params(axis="both", which="major", labelsize=10)

            ax.set_yticks(np.arange(-400, 400 + 1, 400))
            ax.set_xticks(np.arange(-384, 384 + 1, 384))
            ax.set_yticks(np.arange(-400, 400 + 1, 16), minor=True)
            ax.set_xticks(np.arange(-384, 384 + 1, 16), minor=True)

            ax.grid(which="major", color="black", linestyle="solid", linewidth=0.5)

            ax.text(
                -415,
                200,
                "FE1: " + fe_serialNumbers[0],
                horizontalalignment="center",
                verticalalignment="center",
                size=12,
                color="blue",
                rotation=-90,
            )
            ax.text(
                -415,
                -200,
                "FE2: " + fe_serialNumbers[1],
                horizontalalignment="center",
                verticalalignment="center",
                size=12,
                color="blue",
                rotation=-90,
            )
            ax.text(
                415,
                -200,
                "FE3: " + fe_serialNumbers[2],
                horizontalalignment="center",
                verticalalignment="center",
                size=12,
                color="blue",
                rotation=90,
            )
            ax.text(
                415,
                200,
                "FE4: " + fe_serialNumbers[3],
                horizontalalignment="center",
                verticalalignment="center",
                size=12,
                color="blue",
                rotation=90,
            )

            plt.title(
                "Quad Module: " + serialNumber + " " + plot.get("title"),
                fontsize=18,
            )
            plt.xlabel("Row", fontsize=18)
            plt.ylabel("Column", fontsize=18)
            cb = plt.colorbar(label=data.get("z").get("AxisTitle"))
            for t in cb.ax.get_yticklabels():
                t.set_fontsize(12)

            if failing_pixels:
                plt.scatter(
                    x_low - 384,
                    y_low - 400,
                    s=3,
                    c="black",
                    marker="s",
                    label="Low Occ.",
                )
                plt.scatter(
                    x_high - 384,
                    y_high - 400,
                    s=3,
                    c="fuchsia",
                    marker="s",
                    label="High Occ.",
                )
                plt.legend(fontsize=5)

    elif plot.get("type") == "Histo2d" and plot.get("title") == "InjVcalDiff":
        logger.debug(
            f"{plot.get('data')[0].shape}, {plot.get('data')[1].shape}, {plot.get('data')[2].shape}, {plot.get('data')[2].shape}"
        )
        try:
            module = np.sum([d for k, d in plot.get("data").items()], axis=0)
        except Exception as e:
            logger.info(f"Plotting failed: str{e}")

        ma = np.ma.MaskedArray(module, module <= 0)
        min_nonzero_value = np.ma.min(ma)
        max_nonzero_value = np.ma.max(ma)
        min_max_ratio = max_nonzero_value / (min_nonzero_value + 1.0e-19)

        set_log_scale = min_nonzero_value > 0 and min_max_ratio > 100

        plt.figure(figsize=(12, 9))
        if set_log_scale:
            module += min_nonzero_value * 1.0e-2
            plt.imshow(
                module.T,
                origin="lower",
                cmap="turbo",
                norm=LogNorm(min_nonzero_value * 0.5, max_nonzero_value * 2),
            )
        else:
            plt.imshow(
                module.T, origin="lower", cmap="turbo" if not is_config else "magma"
            )

        ax = plt.gca()
        ax.tick_params(axis="both", which="major", labelsize=10)

        plt.title("Module: " + serialNumber + " " + plot.get("title"), fontsize=18)
        plt.xlabel(xlabel=data.get("x").get("AxisTitle"), fontsize=18)
        plt.ylabel(ylabel=data.get("y").get("AxisTitle"), fontsize=18)
        cb = plt.colorbar(label=data.get("z").get("AxisTitle"))
        for t in cb.ax.get_yticklabels():
            t.set_fontsize(12)

    elif plot.get("type") == "Histo1d":  ## add stats
        plt.figure(figsize=(12, 9))
        plt.title("Module: " + serialNumber + " " + plot.get("title"), fontsize=18)

        for geomId, d in plot.get("data").items():
            try:
                bins = np.linspace(
                    d.get("x").get("Low"),
                    d.get("x").get("High"),
                    d.get("x").get("Bins"),
                )

                fit_label = ""
                if (
                    plot.get("title") == "ThresholdDist-0"
                    or plot.get("title") == "NoiseDist-0"
                ):
                    mu, sigma = fit_Histo1d(d.get("Data"), bins)
                    y = (
                        norm.pdf(bins, mu, sigma)
                        * np.sum(d.get("Data"))
                        * np.diff(bins)[0]
                    )
                    plt.plot(bins, y, "r--")
                    fit_label = f"µ: {mu:.2f}, σ: {sigma:.2f}, "

                plt.hist(
                    bins,
                    len(bins),
                    weights=d.get("Data"),
                    histtype="step",
                    label=f"FE{geomId + 1}: "
                    + fe_serialNumbers[geomId]
                    + f"({fit_label}Uf: {data.get('Underflow')}, Of: {data.get('Overflow')})",
                )
                plt.legend(loc="upper right", fontsize=10)
            except AttributeError:
                continue

            with contextlib.suppress(AttributeError):
                plt.xlabel(d.get("x").get("AxisTitle"), fontsize=18)
            plt.ylabel(d.get("y").get("AxisTitle"), fontsize=18)

        plt.tight_layout()

    return plt



from __future__ import annotations

import typer

import yarrtist
from yarrtist.cli.globals import CONTEXT_SETTINGS
from yarrtist.cli.plot_config import main as plot_config
from yarrtist.cli.plot_connectivity import main as plot_connectivity
from yarrtist.cli.plot_scan import main as plot_scan
from yarrtist.cli.plot_test import main as plot_test

app = typer.Typer(context_settings=CONTEXT_SETTINGS)
app_plots = typer.Typer(context_settings=CONTEXT_SETTINGS)
app.add_typer(app_plots, name="plots")


@app.callback(invoke_without_command=True)
def main(
    version: bool = typer.Option(False, "--version", help="Print the current version."),
    prefix: bool = typer.Option(
        False, "--prefix", help="Print the path prefix for data files."
    ),
) -> None:
    """
    Manage top-level options
    """
    if version:
        typer.echo(f"yarrtist v{yarrtist.__version__}")
        raise typer.Exit()
    if prefix:
        typer.echo(yarrtist.data.resolve())
        raise typer.Exit()


app_plots.command("plot-single-test")(plot_test)
app_plots.command("plot-single-config")(plot_config)
app_plots.command("plot-scan")(plot_scan)
app_plots.command("plot-connectivity")(plot_connectivity)
